# Copyright 2026 EcoFuture Technology Services LLC and contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Generated by Django 4.0.7 on 2022-09-26 12:39

from django.db import migrations

def create_extension(apps, schema_editor):
    schema_editor.execute("CREATE EXTENSION IF NOT EXISTS \"uuid-ossp\";")


def drop_extension(apps, schema_editor):
    schema_editor.execute("DROP EXTENSION IF EXISTS \"uuid-ossp\";")



class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.RunPython(create_extension, reverse_code=drop_extension, atomic=True)
    ]

