# Copyright 2026 EcoFuture Technology Services LLC and contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Generated by Django 5.0.6 on 2024-06-27 14:35

from django.db import migrations, models

import bazis.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('entity', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='parententity',
            name='field',
            field=bazis.core.fields.ChoiceArrayField(base_field=models.CharField(choices=[('first_field', 'First field'), ('second_field', 'Second field'), ('third_field', 'Third field')], max_length=32), blank=True, null=True, size=None, verbose_name='Field'),
        ),
        migrations.AddField(
            model_name='parententity',
            name='state',
            field=models.CharField(choices=[('state_one', 'State one'), ('state_two', 'State two'), ('state_three', 'State three'), ('state_four', 'State four')], default='state_one', max_length=255, verbose_name='State'),
        ),
    ]
